@@ -1,5 +1,5 @@
 /* Popup menus for GNOME
- * 
+ *
  * Copyright (C) 1998 Mark Crichton
  * All rights reserved.
  *
@@ -31,11 +31,10 @@
 static GtkWidget* global_menushell_hack = NULL;
 #define TOPLEVEL_MENUSHELL_KEY "gnome-popup-menu:toplevel_menushell_key"
 
-static GtkWidget* 
+static GtkWidget*
 get_toplevel(GtkWidget* menuitem)
 {
-	return gtk_object_get_data (GTK_OBJECT (menuitem),
-				    TOPLEVEL_MENUSHELL_KEY);
+	return g_object_get_data (G_OBJECT (menuitem), TOPLEVEL_MENUSHELL_KEY);
 }
 
 /* Our custom marshaller for menu items.  We need it so that it can extract the per-attachment
@@ -50,16 +49,16 @@ popup_marshal_func (GtkObject *object, gpointer data, guint n_args, GtkArg *args
 {
 	ActivateFunc func;
 	gpointer user_data;
-	GtkObject *tl;
+	GObject *tl;
 	GtkWidget *for_widget;
 
-	tl = GTK_OBJECT (get_toplevel(GTK_WIDGET (object)));
-	user_data = gtk_object_get_data (tl, "gnome_popup_menu_do_popup_user_data");
-	for_widget = gtk_object_get_data (tl, "gnome_popup_menu_do_popup_for_widget");
+	tl = G_OBJECT (get_toplevel(GTK_WIDGET (object)));
+	user_data = g_object_get_data (tl, "gnome_popup_menu_do_popup_user_data");
+	for_widget = g_object_get_data (tl, "gnome_popup_menu_do_popup_for_widget");
 
-	gtk_object_set_data (GTK_OBJECT (get_toplevel(GTK_WIDGET (object))),
-			     "gnome_popup_menu_active_item",
-			     object);
+	g_object_set_data (G_OBJECT (get_toplevel(GTK_WIDGET (object))),
+			   "gnome_popup_menu_active_item",
+			   object);
 
 	func = (ActivateFunc) data;
 	if (func)
@@ -75,26 +74,25 @@ popup_connect_func (GnomeUIInfo *uiinfo, const char *signal_name, GnomeUIBuilder
 {
 	g_assert (uibdata->is_interp);
 
-	gtk_object_set_data (GTK_OBJECT (uiinfo->widget), 
-			     TOPLEVEL_MENUSHELL_KEY, 
-			     global_menushell_hack);
+	g_object_set_data (G_OBJECT (uiinfo->widget),
+			   TOPLEVEL_MENUSHELL_KEY,
+			   global_menushell_hack);
 
-	gtk_signal_connect (GTK_OBJECT (uiinfo->widget),
-			    signal_name,
-			    GTK_SIGNAL_FUNC (popup_marshal_func),
-			    uiinfo->moreinfo);
+	g_signal_connect (uiinfo->widget, signal_name,
+			  G_CALLBACK (popup_marshal_func),
+			  uiinfo->moreinfo);
 }
 
 /**
  * gnome_popup_menu_new_with_accelgroup
- * @uiinfo:
- * @accelgroup:
+ * @uiinfo: A #GnomeUIInfo array describing the new menu.
+ * @accelgroup: A #GtkAccelGroup describing the accelerator key structure.
  *
- * Creates a popup menu out of the specified uiinfo array.  Use
+ * Creates a popup menu out of the specified @uiinfo array.  Use
  * gnome_popup_menu_do_popup() to pop the menu up, or attach it to a
  * window with gnome_popup_menu_attach().
  *
- * Returns a menu widget
+ * Returns: A menu widget.
  */
 GtkWidget *
 gnome_popup_menu_new_with_accelgroup (GnomeUIInfo *uiinfo,
@@ -125,7 +123,7 @@ gnome_popup_menu_new_with_accelgroup (GnomeUIInfo *uiinfo,
 
 /**
  * gnome_popup_menu_new
- * @uiinfo:
+ * @uiinfo: A #GnomeUIInfo array describing the new menu.
  *
  * This function behaves just like
  * gnome_popup_menu_new_with_accelgroup(), except that it creates an
@@ -133,7 +131,7 @@ gnome_popup_menu_new_with_accelgroup (GnomeUIInfo *uiinfo,
  * gnome_popup_menu_get_accel_group() to get the accelgroup that is
  * created.
  *
- * Returns a menu widget
+ * Returns: A menu widget.
  *
  */
 GtkWidget *
@@ -144,12 +142,13 @@ gnome_popup_menu_new (GnomeUIInfo *uiinfo)
 
 /**
  * gnome_popup_menu_get_accel_group
- * @menu: 
+ * @menu: A menu widget.
  *
- * Returns the accelgroup associated with the specified GtkMenu.  This
- * is the accelgroup that was created by gnome_popup_menu_new().  If
- * you want to specify the accelgroup that the popup menu accelerators
- * use, then use gnome_popup_menu_new_with_accelgroup().
+ * This function is used to retrieve the accelgroup that was created by
+ * gnome_popup_menu_new(). If you want to specify the accelgroup that the popup
+ * menu accelerators use, then use gnome_popup_menu_new_with_accelgroup().
+ *
+ * Returns: The accelgroup associated with the specified #GtkMenu.
  */
 GtkAccelGroup *
 gnome_popup_menu_get_accel_group(GtkMenu *menu)
@@ -164,8 +163,9 @@ gnome_popup_menu_get_accel_group(GtkMenu *menu)
 #endif
 }
 
-/* Callback used when a button is pressed in a widget attached to a popup menu.  It decides whether
- * the menu should be popped up and does the appropriate stuff.
+/* Callback used when a button is pressed in a widget attached to a popup
+ * menu.  It decides whether the menu should be popped up and does the
+ * appropriate stuff.
  */
 static gint
 real_popup_button_pressed (GtkWidget *widget, GdkEventButton *event, gpointer data)
@@ -182,7 +182,7 @@ real_popup_button_pressed (GtkWidget *widget, GdkEventButton *event, gpointer da
 	 *
 	 */
 
-	user_data = gtk_object_get_data (GTK_OBJECT (widget), "gnome_popup_menu_attach_user_data");
+	user_data = g_object_get_data (G_OBJECT (widget), "gnome_popup_menu_attach_user_data");
 
 	gnome_popup_menu_do_popup (popup, NULL, NULL, event, user_data, widget);
 
@@ -195,7 +195,7 @@ popup_button_pressed (GtkWidget *widget, GdkEventButton *event, gpointer data)
 	if (event->button != 3)
 		return FALSE;
 
-	gtk_signal_emit_stop_by_name (GTK_OBJECT (widget), "button_press_event");
+	g_signal_stop_emission_by_name (widget, "button_press_event");
 
 	return real_popup_button_pressed (widget, event, data);
 }
@@ -203,7 +203,7 @@ popup_button_pressed (GtkWidget *widget, GdkEventButton *event, gpointer data)
 static gint
 popup_menu_pressed (GtkWidget *widget, gpointer data)
 {
-	gtk_signal_emit_stop_by_name (GTK_OBJECT (widget), "popup_menu");
+	g_signal_stop_emission_by_name (widget, "popup_menu");
 
 	return real_popup_button_pressed (widget, NULL, data);
 }
@@ -216,29 +216,31 @@ relay_popup_button_pressed (GtkWidget *widget, GdkEventButton *event, gpointer d
 	if (event->button != 3)
 		return FALSE;
 
-	gtk_signal_emit_stop_by_name (GTK_OBJECT (widget), "button_press_event");
+	g_signal_stop_emission_by_name (widget, "button_press_event");
 
 	if (GTK_IS_CONTAINER(widget))
 	  {
 
 	    do {
 	      GList *children;
+	      GList *child;
 
-	      for (children = gtk_container_children(GTK_CONTAINER(widget)), new_widget = NULL;
-		   children; children = children->next)
+	      children = gtk_container_get_children (GTK_CONTAINER (widget));
+
+	      for (child = children, new_widget = NULL; child; child = child->next)
 		{
 		  GtkWidget *cur;
-		  
-		  cur = (GtkWidget *)children->data;
-		  
+
+		  cur = (GtkWidget *)child->data;
+
 		  if(! GTK_WIDGET_NO_WINDOW(cur) )
 		    continue;
-		  
+
 		  if(cur->allocation.x <= event->x
 		     && cur->allocation.y <= event->y
 		     && (cur->allocation.x + cur->allocation.width) > event->x
 		     && (cur->allocation.y + cur->allocation.height) > event->y
-		     && gtk_object_get_data (GTK_OBJECT(cur), "gnome_popup_menu_nowindow"))
+		     && g_object_get_data (G_OBJECT(cur), "gnome_popup_menu_nowindow"))
 		    {
 		      new_widget = cur;
 		      break;
@@ -250,7 +252,7 @@ relay_popup_button_pressed (GtkWidget *widget, GdkEventButton *event, gpointer d
 		break;
 	    } while(widget && GTK_IS_CONTAINER (widget) && GTK_WIDGET_NO_WINDOW(widget));
 
-	    if(!widget || !gtk_object_get_data (GTK_OBJECT(widget), "gnome_popup_menu"))
+	    if(!widget || !g_object_get_data (G_OBJECT(widget), "gnome_popup_menu"))
 	      {
 		return TRUE;
 	      }
@@ -270,14 +272,14 @@ popup_attach_widget_destroyed (GtkWidget *widget, gpointer data)
 
 	popup = data;
 
-	gtk_object_unref (GTK_OBJECT (popup));
+	g_object_unref (G_OBJECT (popup));
 }
 
 /**
  * gnome_popup_menu_attach:
- * @popup:
- * @widget:
- * @user_data:
+ * @popup: A menu widget.
+ * @widget: The widget to attach the popup menu to.
+ * @user_data: Application specific data passed to the callback.
  *
  * Attaches the specified popup menu to the specified widget.  The
  * menu can then be activated by pressing mouse button 3 over the
@@ -295,8 +297,8 @@ popup_attach_widget_destroyed (GtkWidget *widget, gpointer data)
  * widgets it is attached to are destroyed, the popup menu will be
  * destroyed as well.
  *
- * Under the current implementation, setting a popup menu for a NO_WINDOW widget and then reparenting
- * that widget will cause Bad Things to happen.
+ * Under the current implementation, setting a popup menu for a NO_WINDOW
+ * widget and then reparenting that widget will cause Bad Things to happen.
  */
 void
 gnome_popup_menu_attach (GtkWidget *popup, GtkWidget *widget,
@@ -309,30 +311,30 @@ gnome_popup_menu_attach (GtkWidget *popup, GtkWidget *widget,
 	g_return_if_fail (widget != NULL);
 	g_return_if_fail (GTK_IS_WIDGET (widget));
 
-	if(gtk_object_get_data (GTK_OBJECT (widget), "gnome_popup_menu"))
+	if(g_object_get_data (G_OBJECT (widget), "gnome_popup_menu"))
 	  return;
 
-	gtk_object_set_data (GTK_OBJECT (widget), "gnome_popup_menu", popup);
+	g_object_set_data (G_OBJECT (widget), "gnome_popup_menu", popup);
 
 	/* This operation can fail if someone is trying to set a popup on e.g. an uncontained label, so we do it first. */
 	for(ev_widget = widget; ev_widget && GTK_WIDGET_NO_WINDOW(ev_widget); ev_widget = ev_widget->parent)
 	  {
-	    gtk_object_set_data (GTK_OBJECT (ev_widget), "gnome_popup_menu_nowindow", GUINT_TO_POINTER(1));
+	    g_object_set_data (G_OBJECT (ev_widget), "gnome_popup_menu_nowindow", GUINT_TO_POINTER(1));
 	  }
 
 	g_return_if_fail (ev_widget);
 
 	/* Ref/sink the popup menu so that we take "ownership" of it */
 
-	gtk_object_ref (GTK_OBJECT (popup));
+	g_object_ref (G_OBJECT (popup));
 	gtk_object_sink (GTK_OBJECT (popup));
 
 	/* Store the user data pointer in the widget -- we will use it later when the menu has to be
 	 * invoked.
 	 */
 
-	gtk_object_set_data (GTK_OBJECT (widget), "gnome_popup_menu_attach_user_data", user_data);
-	gtk_object_set_data (GTK_OBJECT (widget), "gnome_popup_menu", user_data);
+	g_object_set_data (G_OBJECT (widget), "gnome_popup_menu_attach_user_data", user_data);
+	g_object_set_data (G_OBJECT (widget), "gnome_popup_menu", user_data);
 
 	/* Prepare the widget to accept button presses -- the proper assertions will be
 	 * shouted by gtk_widget_set_events().
@@ -341,33 +343,40 @@ gnome_popup_menu_attach (GtkWidget *popup, GtkWidget *widget,
 	gtk_widget_add_events (ev_widget, GDK_BUTTON_PRESS_MASK |
 			       GDK_KEY_PRESS_MASK);
 
-	gtk_signal_connect (GTK_OBJECT (widget), "button_press_event",
-			    GTK_SIGNAL_FUNC (popup_button_pressed), popup);
+	g_signal_connect (widget, "button_press_event",
+			  G_CALLBACK (popup_button_pressed), popup);
 
 	g_signal_connect (G_OBJECT (widget), "popup_menu",
 			  G_CALLBACK (popup_menu_pressed), popup);
-	
-	if (ev_widget != widget)
-	  gtk_signal_connect_while_alive (GTK_OBJECT (ev_widget), "button_press_event",
-					  (GtkSignalFunc) relay_popup_button_pressed,
-					  popup, GTK_OBJECT(widget));
+
+	if (ev_widget != widget) {
+		GClosure *closure;
+
+		closure = g_cclosure_new (G_CALLBACK (relay_popup_button_pressed),
+					  popup,
+					  NULL);
+		g_object_watch_closure (G_OBJECT (widget), closure);
+		g_signal_connect_closure (ev_widget, "button_press_event",
+					  closure, FALSE);
+	}
 
 	/* This callback will unref the popup menu when the widget it is attached to gets destroyed. */
-	gtk_signal_connect (GTK_OBJECT (widget), "destroy",
-			    (GtkSignalFunc) popup_attach_widget_destroyed,
-			    popup);
+	g_signal_connect (widget, "destroy",
+			  G_CALLBACK (popup_attach_widget_destroyed),
+			  popup);
 }
 
 /**
  * gnome_popup_menu_do_popup:
- * @popup:
- * @pos_func:
- * @pos_data:
- * @event:
- * @user_data:
+ * @popup: A menu widget.
+ * @pos_func: A user supplied function to position the menu or %NULL.
+ * @pos_data: User supplied data to pass to @pos_func.
+ * @event: A #GdkEventButton structure containing the event that triggered the
+ * menu (or %NULL).
+ * @user_data: Application specific data passed to the menu callback.
+ * @for_widget: The widget for which the popup was triggered.
  *
- *
- * You can use this function to pop up a menu.  When a menu item *
+ * You can use this function to pop up a menu.  When a menu item
  * callback is invoked, the specified user_data will be passed to it.
  *
  * The pos_func and pos_data parameters are the same as for
@@ -395,8 +404,8 @@ gnome_popup_menu_do_popup (GtkWidget *popup, GtkMenuPositionFunc pos_func, gpoin
 	 * marshaller.
 	 */
 
-	gtk_object_set_data (GTK_OBJECT (popup), "gnome_popup_menu_do_popup_user_data", user_data);
-	gtk_object_set_data (GTK_OBJECT (popup), "gnome_popup_menu_do_popup_for_widget", for_widget);
+	g_object_set_data (G_OBJECT (popup), "gnome_popup_menu_do_popup_user_data", user_data);
+	g_object_set_data (G_OBJECT (popup), "gnome_popup_menu_do_popup_for_widget", for_widget);
 
 	if (event) {
 		button = event->button;
@@ -424,7 +433,7 @@ get_active_index (GtkMenu *menu)
 	GtkWidget *active;
 	int i;
 
-	active = gtk_object_get_data (GTK_OBJECT (menu), "gnome_popup_menu_active_item");
+	active = g_object_get_data (G_OBJECT (menu), "gnome_popup_menu_active_item");
 
 	for (i = 0, l = GTK_MENU_SHELL (menu)->children; l; l = l->next, i++)
 		if (active == l->data)
@@ -435,14 +444,17 @@ get_active_index (GtkMenu *menu)
 
 /**
  * gnome_popup_menu_do_popup_modal:
- * @popup:
- * @pos_func:
- * @pos_data:
- * @event:
- * @user_data:
- * 
- * Same as above, but runs the popup menu modally and returns the
- * index of the selected item, or -1 if none.
+ * @popup: A menu widget.
+ * @pos_func: A user supplied function to position the menu or %NULL.
+ * @pos_data: User supplied data to pass to @pos_func.
+ * @event: A #GdkEventButton structure containing the event that triggered the
+ * menu (or %NULL).
+ * @user_data: Application specific data passed to the menu callback.
+ * @for_widget: The widget for which the popup was triggered.
+ *
+ * Same as gnome_popup_do_modal(), but runs the popup menu modally.
+ *
+ * Returns: The index of the selected item or -1 if no item selected.
  */
 int
 gnome_popup_menu_do_popup_modal (GtkWidget *popup, GtkMenuPositionFunc pos_func, gpointer pos_data,
@@ -457,13 +469,13 @@ gnome_popup_menu_do_popup_modal (GtkWidget *popup, GtkMenuPositionFunc pos_func,
 
 	/* Connect to the deactivation signal to be able to quit our modal main loop */
 
-	id = gtk_signal_connect (GTK_OBJECT (popup), "deactivate",
-				 (GtkSignalFunc) menu_shell_deactivated,
-				 NULL);
+	id = g_signal_connect (popup, "deactivate",
+			       G_CALLBACK (menu_shell_deactivated),
+			       NULL);
 
-	gtk_object_set_data (GTK_OBJECT (popup), "gnome_popup_menu_active_item", NULL);
-	gtk_object_set_data (GTK_OBJECT (popup), "gnome_popup_menu_do_popup_user_data", user_data);
-	gtk_object_set_data (GTK_OBJECT (popup), "gnome_popup_menu_do_popup_for_widget", for_widget);
+	g_object_set_data (G_OBJECT (popup), "gnome_popup_menu_active_item", NULL);
+	g_object_set_data (G_OBJECT (popup), "gnome_popup_menu_do_popup_user_data", user_data);
+	g_object_set_data (G_OBJECT (popup), "gnome_popup_menu_do_popup_for_widget", for_widget);
 
 	if (event) {
 		button = event->button;
@@ -478,11 +490,18 @@ gnome_popup_menu_do_popup_modal (GtkWidget *popup, GtkMenuPositionFunc pos_func,
 	gtk_main ();
 	gtk_grab_remove (popup);
 
-	gtk_signal_disconnect (GTK_OBJECT (popup), id);
+	g_signal_handler_disconnect (G_OBJECT (popup), id);
 
 	return get_active_index (GTK_MENU (popup));
 }
 
+/**
+ * gnome_popup_menu_append:
+ * @popup: An existing popup menu.
+ * @uiinfo: A #GnomeUIInfo array describing new menu elements.
+ *
+ * Appends the menu items in @uiinfo the @popup menu.
+ */
 void
 gnome_popup_menu_append (GtkWidget *popup, GnomeUIInfo *uiinfo)
 {
@@ -510,19 +529,19 @@ gnome_popup_menu_append (GtkWidget *popup, GnomeUIInfo *uiinfo)
 
 /**
  * gnome_gtk_widget_add_popup_items:
- * @widget: The widget to append popup menu items for
- * @uiinfo: The array holding information on the menu items
- * @user_data: The user_data to pass to the callbacks for the menu items
+ * @widget: The widget to append popup menu items for.
+ * @uiinfo: A #GnomeUIInfo array describing new menu elements.
+ * @user_data: The user_data to pass to the callbacks for the menu items.
  *
- * This creates a new popup menu for the widget if none exists, and
- * then adds the items in 'uiinfo' to the widget's popup menu.
+ * This creates a new popup menu for @widget if none exists, and then adds the
+ * items in @uiinfo to the popup menu of @widget.
  */
 void
 gnome_gtk_widget_add_popup_items (GtkWidget *widget, GnomeUIInfo *uiinfo, gpointer user_data)
 {
   GtkWidget *prev_popup;
 
-  prev_popup = gtk_object_get_data (GTK_OBJECT (widget), "gnome_popup_menu");
+  prev_popup = g_object_get_data (G_OBJECT (widget), "gnome_popup_menu");
 
   if(!prev_popup)
     {
